<?php

namespace OrganizationsManagement\Controller;

use Exception;
use OrganizationsManagement\Traits\TResponse;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DTO;
use OrganizationsManagement\Service\Notice as NoticeService;

class Notice extends Simple
{

    use TResponse;

    /**
     * @var NoticeService
     */
    private $noticeService;

    /**
     * Notice constructor.
     * @param NoticeService
     */
    public function __construct(NoticeService $noticeService)
    {
        $this->noticeService = $noticeService;
    }

    public function findNotice(DTO\Request $request, DTO\Response $response)
    {
        try {
            $data = $this->noticeService->findNotice();
            $this->sendResponse($data, 200);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function createNotice(DTO\Request $request, DTO\Response $response)
    {
        try {

            $data = $request->getParameters();
            $this->noticeService->createNotice($data);
            $this->sendResponse(['success' => 'Comunicado adicionado com sucesso!'], 200);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function deleteNotice(DTO\Request $request, DTO\Response $response)
    {
        try {
            $params = $request->getParameters();
            $this->noticeService->deleteNotice($params['id']);
            $this->sendResponse(['success' => 'Comunicado(s) excluído(s) com sucesso!'], 200);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function updateNotice(DTO\Request $request, DTO\Response $response)
    {
        try {
            $data = $request->getParameters();
            $this->noticeService->updateNotice($data);
            $this->sendResponse(['success' => 'Comunicado(s) atualizado(s) com sucesso!'], 200);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function findTimeOutnotice(DTO\Request $request, DTO\Response $response)
    {
        try {
            $data = $this->noticeService->findTimeOutnotice();
            $this->sendResponse($data, 200);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function saveTimeOutNotice(DTO\Request $request)
    {
        try {
            $this->noticeService->saveTimeOutNotice($request->getParameters());
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function deleteTimeOutNotice(DTO\Request $request)
    {
        try {
            $this->noticeService->deleteTimeOutNotice($request->getParameters());
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }
    


    /**
     * @param DTO\Request $request
     * @param DTO\Response $response
     * @throws Exception
     */
}
