<?php


namespace OrganizationsManagement\Repository;


use Doctrine\ORM\EntityManager;
use OrganizationsManagement\Util\InstanceProvider;
use Teknisa\Model\Traits\TQuery;

class RedirecionamentoAssinatura
{
    use TQuery;
    /**
     * @var EntityManager
     */
    private $entityManager;

    /**
     * RedirecionamentoAssinaturaRepository constructor.
     * @param EntityManager $entityManager
     */
    public function __construct(EntityManager $entityManager)
    {
        $this->entityManager = $entityManager;
    }

    // TO_CHAR(SYSDATE, 'DD/MM/YYYY')

    public function getParamCentrAssinatura()
    {
        $query = "SELECT * FROM PARAM_CENTR_ASSINATURA";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $query);
    } 

    public function getTipoDocCertsign()
    {
        $query = "SELECT
                        NRTIPODOCCERTSIGN,
                        NMTIPODOCCERTSIGN
                    FROM
                        FCT_TIPODOCCERTSIGN
                    WHERE
                        IDATIVO = 'S'";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $query);
    }

    public function getRotasCentrAssinatura()
    {
        $query = "SELECT
                        ID,
                        DSCHAVEASSINAT,
                        DSROTAASSINAT,
                        DSURLBASE,
                        CLIENT_ID,
                        CLIENT_SECRET
                    FROM
                        ROTAS_CENTR_ASSINATURA";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $query);
    }

    public function getLogCentrAssinatura()
    {
        $query = "SELECT
                        LOGCENTR.ID,
                        LOGCENTR.ID_CENTR_ASSINA,
                        CENTR.IDDOCASSINAT,
                        LOGCENTR.NRCPFSIGNATARIO,
                        DECODE(LOGCENTR.IDACEITARECUSA, 'A', 'Aceito', 'Recusado') as IDACEITARECUSA,
                        LOGCENTR.DTINCLUSAO as DTASSINATURA,
                        LOGCENTR.DSLOG,
                        ROTA.DSCHAVEASSINAT,
                        ROTA.DSROTAASSINAT,
                        ROTA.DSURLBASE,
                        ROTA.CLIENT_ID,
                        ROTA.CLIENT_SECRET
                    FROM
                            LOG_CENTR_ASSINATURA LOGCENTR
                        INNER JOIN CENTRALIZA_ASSINATURA  CENTR 
                            ON CENTR.ID = LOGCENTR.ID_CENTR_ASSINA
                        INNER JOIN ROTAS_CENTR_ASSINATURA ROTA 
                            ON CENTR.DSCHAVEASSINAT = ROTA.DSCHAVEASSINAT";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $query);
    }

    public function updateParamCentrAssinatura($data, $operador, $nrorg)
    {
        $connection = InstanceProvider::getEntityManager()->getConnection();
        $connection->beginTransaction();
        try {

            $query = "UPDATE PARAM_CENTR_ASSINATURA 
                        SET DSTOKENCERTSIGN = :DSTOKENCERTSIGN,
                            NMSENDERCERTSIGN = :NMSENDERCERTSIGN,
                            DSEMAILSENDERCERTSIGN = :DSEMAILSENDERCERTSIGN,
                            NRCPFSENDERCERTSIGN = :NRCPFSENDERCERTSIGN,
                            IDPADRAOASSINACERTSIGN = :IDPADRAOASSINACERTSIGN,
                            NRTIPODOCCERTSIGN = :NRTIPODOCCERTSIGN,
                            QTDIALEMBRETE = :QTDIALEMBRETE,
                            DTULTATU = SYSDATE,
                            CDOPERULTATU = :CDOPERULTATU,
                            NRORGULTATU = :NRORGULTATU 
                    WHERE ID = :ID";
            
            $params = array(
                'ID'                        => $data['ID'],
                'DSTOKENCERTSIGN'           => $data['DSTOKENCERTSIGN'],
                'NMSENDERCERTSIGN'          => $data['NMSENDERCERTSIGN'],
                'DSEMAILSENDERCERTSIGN'     => $data['DSEMAILSENDERCERTSIGN'],
                'NRCPFSENDERCERTSIGN'       => $data['NRCPFSENDERCERTSIGN'],
                'IDPADRAOASSINACERTSIGN'    => $data['IDPADRAOASSINACERTSIGN'],
                'NRTIPODOCCERTSIGN'         => $data['NRTIPODOCCERTSIGN'],
                'QTDIALEMBRETE'             => $data['QTDIALEMBRETE'],
                'CDOPERULTATU'              => $operador,
                'NRORGULTATU'               => $nrorg,
            );

            $connection->executeQuery($query, $params);
                 
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw new \Exception($e->getMessage());
        }
    }

    public function validaRotaExistente($params)
    {
        $query = "SELECT
                        COUNT(*) AS VALIDA
                    FROM
                        ROTAS_CENTR_ASSINATURA
                    WHERE
                            UPPER (DSCHAVEASSINAT) = UPPER (:DSCHAVEASSINAT)
                        AND NRORG = :NRORG
                        AND ID <> :ID";

        $params = array(
            'ID'                => $params['ID'],
            'DSCHAVEASSINAT'    => $params['DSCHAVEASSINAT'],
            'NRORG'             => $params['NRORG'],
        );

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $query, $params);
    }

    public function insertRotaAssinatura($rota, $operador, $nrorg, $newId)
    {
        $connection = InstanceProvider::getEntityManager()->getConnection();
        $connection->beginTransaction();

        try {

            $paramsValidacao = array(
                'ID'                => $newId,
                'DSCHAVEASSINAT'    => $rota['DSCHAVEASSINAT'],
                'NRORG'             => $nrorg,
            );

            $existe = $this->validaRotaExistente($paramsValidacao);
            
            if ($existe["data"][0]["VALIDA"] > 0) {
                throw new \Exception("A rotina informada já existe na lista de Rotas. Por favor, verifique os dados inseridos.");
            }
            
            $query = "INSERT
              INTO ROTAS_CENTR_ASSINATURA
                   (ID,
                    DSCHAVEASSINAT,
                    DSROTAASSINAT,
                    NRORG,
                    IDATIVO,
                    DTINCLUSAO,
                    CDOPERINCLUSAO,
                    NRORGINCLUSAO,
                    DSURLBASE,
                    CLIENT_ID,
                    CLIENT_SECRET) 
            VALUES (:ID,
                    :DSCHAVEASSINAT,
                    :DSROTAASSINAT,
                    :NRORG,
                    :IDATIVO,
                    SYSDATE,
                    :CDOPERINCLUSAO,
                    :NRORGINCLUSAO,
                    :DSURLBASE,
                    :CLIENT_ID,
                    :CLIENT_SECRET)";
            
            $params = array(
                'ID'                => $newId,
                'DSCHAVEASSINAT'    => $rota['DSCHAVEASSINAT'],
                'DSROTAASSINAT'     => $rota['DSROTAASSINAT'],
                'NRORG'             => $nrorg,
                'IDATIVO'           => 'S',
                'CDOPERINCLUSAO'    => $operador,
                'NRORGINCLUSAO'     => $nrorg,
                'DSURLBASE'         => $rota['DSURLBASE'],
                'CLIENT_ID'         => $rota['CLIENT_ID'],
                'CLIENT_SECRET'     => $rota['CLIENT_SECRET'],
            );

            $connection->executeQuery($query, $params);
                 
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw new \Exception($e->getMessage());
        }
    }

    public function updateRotaAssinatura($rota, $operador, $nrorg)
    {
        $connection = InstanceProvider::getEntityManager()->getConnection();
        $connection->beginTransaction();

        try {

            $paramsValidacao = array(
                'ID'                => $rota['ID'],
                'DSCHAVEASSINAT'    => $rota['DSCHAVEASSINAT'],
                'NRORG'             => $nrorg,
            );

            $existe = $this->validaRotaExistente($paramsValidacao);
            
            if ($existe["data"][0]["VALIDA"] > 0) {
                throw new \Exception("A rotina informada já existe na lista de Rotas. Por favor, verifique os dados inseridos.");
            }

            $query = "UPDATE ROTAS_CENTR_ASSINATURA 
                        SET DSCHAVEASSINAT = :DSCHAVEASSINAT,
                            DSROTAASSINAT = :DSROTAASSINAT,
                            NRORG = :NRORG,
                            IDATIVO = :IDATIVO,
                            DTULTATU = SYSDATE,
                            CDOPERULTATU = :CDOPERULTATU,
                            NRORGULTATU = :NRORGULTATU,
                            DSURLBASE = :DSURLBASE,
                            CLIENT_ID = :CLIENT_ID,
                            CLIENT_SECRET = :CLIENT_SECRET
                        WHERE ID = :ID";
            
            $params = array(
                'ID'                => $rota['ID'],
                'DSCHAVEASSINAT'    => $rota['DSCHAVEASSINAT'],
                'DSROTAASSINAT'     => $rota['DSROTAASSINAT'],
                'NRORG'             => $nrorg,
                'IDATIVO'           => 'S',
                'CDOPERULTATU'      => $operador,
                'NRORGULTATU'       => $nrorg,
                'DSURLBASE'         => $rota['DSURLBASE'],
                'CLIENT_ID'         => $rota['CLIENT_ID'],
                'CLIENT_SECRET'     => $rota['CLIENT_SECRET'],
            );

            $connection->executeQuery($query, $params);
                 
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw new \Exception($e->getMessage());
        }
    }

    public function deleteRotaAssinatura($rows)
    {
        $connection = InstanceProvider::getEntityManager()->getConnection();
        $connection->beginTransaction();
        try {
            foreach ($rows as $rota) {
                $query = "DELETE
                            FROM ROTAS_CENTR_ASSINATURA
                            WHERE ID = :ID";
                
                $params = array(
                    'ID' => $rota['ID'],
                );
                
                $connection->executeQuery($query, $params);
            }
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw new \Exception($e->getMessage());
        }
    }
}
