<?php

namespace OrganizationsManagement\Service;

use Doctrine\ORM\EntityManager;
use OrganizationsManagement\Repository\CollectionErrors as CollectionErrorsRepository;
use OrganizationsManagement\Util\InstanceProvider;
use Teknisa\Libs\Util\Environment;

class CollectionErrors
{
    /**
     * @var CollectionErrorsRepository
     */
    private $collectionErrorsRepository;

    /**
     * @param CollectionErrorsRepository $collectionErrorsRepository
     */
    public function __construct(CollectionErrorsRepository $collectionErrorsRepository, Environment $environment)
    {
        $this->collectionErrorsRepository = $collectionErrorsRepository;
        $this->environment = $environment; 
    }

    public function find($page, $limit, $order, $search, $searchIn, $dynamicFilter)
    {
        $nrorg = $this->environment->getNrorg();

        $dynamicFilter = json_decode(utf8_encode(base64_decode($dynamicFilter)),true);

        $parametros = array(
            "DTINICOMPET"       => $dynamicFilter["DTINICOMPET"][0]['value'],
            "DTFINCOMPET"       => $dynamicFilter["DTFINCOMPET"][0]['value'],
            "NMPARCEIRO"        => isset($dynamicFilter["NMPARCEIRO"][0]['value'])? $dynamicFilter["NMPARCEIRO"][0]['value'] : null,
            "DSTPMEDIDASERV"    => isset($dynamicFilter["DSTPMEDIDASERV"][0]['value'])? $dynamicFilter["DSTPMEDIDASERV"][0]['value'] : null,
            "NMELEMENTO"        => isset($dynamicFilter["NMELEMENTO"][0]['value'])? $dynamicFilter["NMELEMENTO"][0]['value'] : null,
            "NMSERVORCA"        => isset($dynamicFilter["NMSERVORCA"][0]['value'])? $dynamicFilter["NMSERVORCA"][0]['value'] : null,
            "IDTIPOORG"         => isset($dynamicFilter["IDTIPOORG"][0]['value'])? $dynamicFilter["IDTIPOORG"][0]['value'] : null,
            "CONTRATO"          => isset($dynamicFilter["CONTRATO"][0]['value'])? $dynamicFilter["CONTRATO"][0]['value'] : null,
            "IDSTATUS"          => isset($dynamicFilter["IDSTATUS"][0]['value'])? $dynamicFilter["IDSTATUS"][0]['value'] : null,
            "IDTPCOLETA"        => isset($dynamicFilter["IDTPCOLETA"][0]['value'])? $dynamicFilter["IDTPCOLETA"][0]['value'] : null,
            "IDSEMERRO"         => isset($dynamicFilter["IDSEMERRO"][0]['value'])? $dynamicFilter["IDSEMERRO"][0]['value'] : null,
            "DSOBSCOLETA"       => isset($dynamicFilter["DSOBSCOLETA"][0]['value'])? $dynamicFilter["DSOBSCOLETA"][0]['value'] : null,
            "NRORG"             => $nrorg
        );

        $filtro = ' ';

        $filtroParceiro = '';
        if (isset($parametros["NMPARCEIRO"])) {
            $nmParceiro = strtoupper($parametros["NMPARCEIRO"]);
            $filtroParceiro = "AND UPPER(NMPARCEIRO) LIKE '%" . $nmParceiro . "%'";
        }

        $filtro .= $filtroParceiro;

        $filtroMedidaServ = '';
        if (isset($parametros["DSTPMEDIDASERV"])) {
            $dsTpMedidaServ = strtoupper($parametros["DSTPMEDIDASERV"]);
            $filtroMedidaServ = " AND UPPER(DSTPMEDIDASERV) LIKE '%" . $dsTpMedidaServ . "%'";
        }

        $filtro .= $filtroMedidaServ;

        $filtroNmElemento = '';
        if (isset($parametros["NMELEMENTO"])) {
            $dsNmElemento = strtoupper($parametros["NMELEMENTO"]);
            $filtroNmElemento = " AND UPPER(NMELEMENTO) LIKE '%" . $dsNmElemento . "%'";
        }

        $filtro .= $filtroNmElemento;

        $filtroNmServOrca = '';
        if (isset($parametros["NMSERVORCA"])) {
            $dsNmServOrca = strtoupper($parametros["NMSERVORCA"]);
            $filtroNmServOrca = " AND UPPER(NMSERVORCA) LIKE '%" . $dsNmServOrca . "%'";
        }

        $filtro .= $filtroNmServOrca;

        $filtroIdTipoOrg = '';
        if (isset($parametros["IDTIPOORG"])) {
            $arrayIdTipoOrg = $this->formatarArray($parametros["IDTIPOORG"]);
            $filtroIdTipoOrg = " AND IDTIPOORG IN (" . $arrayIdTipoOrg . ")";
        }

        $filtro .= $filtroIdTipoOrg;

        $filtroContrato = '';
        if (isset($parametros["CONTRATO"])) {
            $filtroContrato = " AND CONTRATO LIKE '%" . $parametros["CONTRATO"] . "%'";
        }

        $filtro .= $filtroContrato;

        $filtroIdStatus = '';
        if (isset($parametros["IDSTATUS"])) {
            $arrayIdStatus = $this->formatarArray($parametros["IDSTATUS"]);
            $filtroIdStatus = " AND IDSTATUS IN (" . $arrayIdStatus . ")";
        }

        $filtro .= $filtroIdStatus;

        $filtroIdTpColeta = '';
        if (isset($parametros["IDTPCOLETA"])) {
            $arrayIdTpColeta = $this->formatarArray($parametros["IDTPCOLETA"]);
            $filtroIdTpColeta = " AND IDTPCOLETA IN (" . $arrayIdTpColeta . ")";
        }

        $filtro .= $filtroIdTpColeta;

        $filtroIdSemErro = '';
        if (isset($parametros["IDSEMERRO"])) {
            $filtroIdSemErro = " AND IDSEMERRO = '" . $parametros["IDSEMERRO"] . "' ";
        }

        $filtro .= $filtroIdSemErro;

        $filtroDsObsColeta = '';
        if (isset($parametros["DSOBSCOLETA"])) {
            $filtroDsObsColeta = " AND DSOBSCOLETA LIKE '%" . $parametros["DSOBSCOLETA"] . "%'";
        }

        $filtro .= $filtroDsObsColeta;

        $order = empty($order) ? '' : $this->collectionErrorsRepository->createOrderBy($order);

        return $this->collectionErrorsRepository->getCollectionErros($parametros, $page, $limit, $order, $filtro);
    }

    private function formatarArray($array)
    {
        $arrayModificado = array();

        foreach ($array as $valor) {
            array_push($arrayModificado, "'" . $valor . "'");
        }

        return implode(',', $arrayModificado);
    }
}
