<?php
namespace OrganizationsManagement\Service;

use Doctrine\ORM\EntityManager;
use OrganizationsManagement\Util\InstanceProvider;
use OrganizationsManagement\Repository\OqueHaDeNovo as OqueHaDeNovoRepository;
use OrganizationsManagement\Util\Utilities;
use Zeedhi\Framework\Remote\FileUploader\Server;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\Remote\HttpInterface;

class OqueHaDeNovo {
    /**
     * @var EntityManager
     */
    private $entityManager;
    /**
     * @var OqueHaDeNovoRepository
     */
    private $oqueHaDeNovoRepository;

    private $fileServerFilesUrl;

    /**
     * Classification constructor.
     * @param EntityManager $entityManager
     * @param CadastroAssuntoRepository $cadastroAssuntoRepository
     */
    public function __construct(EntityManager $entityManager, OqueHaDeNovoRepository $oqueHaDeNovoRepository, $fileServerFilesUrl)
    {
        $this->entityManager = $entityManager;
        $this->oqueHaDeNovoRepository = $oqueHaDeNovoRepository;
        $this->fileServerFilesUrl = $fileServerFilesUrl;
    }

    public function findItemProduct($dynamicFilter)
    {
        $dtfiltroitemString = '';
        $assuntoIdString = '';
        $conditionsQuery = '';
        $dynamicFilter = Utilities::decodificarDynamicFilter($dynamicFilter);
        if (!empty($dynamicFilter["DTFILTROITEM"])) {
            $dates = $dynamicFilter["DTFILTROITEM"][0]["value"];
            $dateIni = $dates[0];
            $dateEnd = $dates[1];
            $dtfiltroitemString = " AND (  ( TO_DATE(DTFILTROITEM, 'DD/MM/YYYY HH24:MI:SS')  BETWEEN  TO_DATE('$dateIni 00:00:00', 'DD/MM/YYYY HH24:MI:SS') AND TO_DATE('$dateEnd 23:59:59', 'DD/MM/YYYY HH24:MI:SS') ) OR DTFILTROITEM IS NULL)";
            unset($dynamicFilter["DTFILTROITEM"]);
        }
        if (!empty($dynamicFilter["ASSUNTO_ID"])) {
            $filterAssunto = array (
                "ASSUNTO_ID" => $dynamicFilter["ASSUNTO_ID"],
            );
            $filterTempAssunto = $this->oqueHaDeNovoRepository->createDynamicFilter($filterAssunto, null, false);
            $assuntoIdString = "AND T.ID IN (SELECT ITEM_ID FROM OM_OQA_ITEM_ASSUNTO $filterTempAssunto)";
            unset($dynamicFilter["ASSUNTO_ID"]);
        }
        if (!empty($dynamicFilter)) {
            $conditionsQuery = $this->oqueHaDeNovoRepository->createDynamicFilter($dynamicFilter, 'WHERE 1=1',false);
        }
        $data =  $this->oqueHaDeNovoRepository->findItemProduct($dtfiltroitemString, $conditionsQuery, $assuntoIdString);
        $data['data'] = Utilities::fixParentsForTreeGrid($data['data'], 'TREE_ID');
        return $data;
    }

    public function findOmProduct()
    {
        return $this->oqueHaDeNovoRepository->findOmProduct();
    }

    public function findTreeGridItem($parametros)
    {
        $produtoId = null;
        if (sizeof($parametros) > 0) {
            $produtoId = $parametros["PRODUTO_ID"];
        }

        $fetchOnDemand = '';
        if ($produtoId != 'NULL' ) {
            $fetchOnDemand = " AND PRODUTO_ID = '" . $produtoId . "'";
        }

        $fetchOnDemandPrimeiraLinha = '';
        if ($produtoId == 'NULL') {
            $fetchOnDemandPrimeiraLinha = " AND PRODUTO_ID IS NULL";
        }
        return $this->oqueHaDeNovoRepository->findTreeGridItem($fetchOnDemand, $fetchOnDemandPrimeiraLinha);
    }

    public function saveModalAddOqueHaDeNovo($dsassunto, $assuntos, $produtoId, $paiId, $document, $dtfiltroitem) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try{
            $typeFile = null;
            $labelFile = null;
            $sizeFile = null;
            $pathFile = null;
            if (!empty($document)) {
                $typeFile = $document[0]['type'];
                $labelFile = $document[0]['name'];
                $sizeFile = $document[0]['size'];
                $pathFile = $document[0]['path'];
            }
            if ($dtfiltroitem !== null) {
                $dtfiltroitem = \DateTime::createFromFormat('d/m/Y', $dtfiltroitem);
            }
            $omOQAItem = InstanceProvider::getFactory()->getCadItem()->createItem($dsassunto, $produtoId, $paiId, $pathFile, $typeFile, $sizeFile , $labelFile, $dtfiltroitem);
            $this->entityManager->persist($omOQAItem);
            $this->entityManager->flush();
            if ($assuntos !== null) {
                foreach ($assuntos as $assunto) {
                    $omOQAAssunto = InstanceProvider::getRepository()->getOmOqaAssunto()->findOneBy(array('id' => $assunto));
                    $assuntos = $this->getPaiIdAssuntos($omOQAAssunto->getPaiId(), $assuntos);
                }
                $assuntos = array_unique($assuntos);
                foreach ($assuntos as $assunto) {
                    $omOQAItemAssunto = InstanceProvider::getFactory()->getCadItem()->createItemAssunto($omOQAItem->getId(), $assunto);
                    $this->entityManager->persist($omOQAItemAssunto);
                }
                $this->atualizaAssuntosPai($paiId, $assuntos);
                $this->entityManager->flush();
            }
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function getPaiIdAssuntos ($paiId, $assuntos) {
        $omOQAAssunto = InstanceProvider::getRepository()->getOmOqaAssunto()->findOneBy(array('id' => $paiId));
        if (!empty($omOQAAssunto) && !in_array($omOQAAssunto->getId(), $assuntos)) {
            $assuntos[] = $omOQAAssunto->getId();
            $assuntos = $this->getPaiIdAssuntos($omOQAAssunto->getPaiId(), $assuntos);
        }
        return $assuntos;
    }

    public function atualizaAssuntosPai($paiId, $assuntos) {
        if (!empty($paiId) && !empty($assuntos)) {
            /** @var \Teknisa\Model\Teknisa\Entities\OmOqaItem $omOQAItem */
            $omOQAItem = InstanceProvider::getRepository()->getOmOqaItem()->findOneBy(array('id' => $paiId));
            $omOQAItemAssunto = InstanceProvider::getRepository()->getOmOqaItemAssunto()->findBy(array('assuntoId' => $assuntos, 'itemId' => $paiId));
            $idsAssuntoPai = Utilities::getArrayByArrayObjectWithGet($omOQAItemAssunto, ['getAssuntoId']);
            foreach ($assuntos as $assunto) {
                if (!in_array($assunto, $idsAssuntoPai)) {
                    $omOQAItemAssunto = InstanceProvider::getFactory()->getCadItem()->createItemAssunto($omOQAItem->getId(), $assunto);
                    $this->entityManager->persist($omOQAItemAssunto);
                }
            }
            $this->atualizaAssuntosPai($omOQAItem->getPaiId(), $assuntos);
        }
    }

    public function getPathOqueHaDeNovo($pdfPath)
    {   
        if ($pdfPath == null) {
            return $data[] = array();    
        }
        return $data[] = array(
            'pdfPath' => $this->fileServerFilesUrl . $pdfPath,
        );
    }

    public function getSubjectsItem($id)
    {   
        $omOQAItemAssuntos = InstanceProvider::getRepository()->getOmOqaItemAssunto()->findBy(array('itemId' => $id));
        $arrayAssuntoId = array();
        foreach ($omOQAItemAssuntos as $omOQAItemAssunto) {
            array_push($arrayAssuntoId, $omOQAItemAssunto->getAssuntoId());
        }
        return $arrayAssuntoId;
    }

    public function getPathFileItem($id)
    {   
        $omOQAItemRepository = InstanceProvider::getRepository()->getOmOqaItem()->findOneBy(array('id' => $id));
        return $this->getPathOqueHaDeNovo($omOQAItemRepository->getFilename());
    }

    public function updateItemOqueHaDeNovo($data) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try{
            $id = $data['ID'];
            $dsassunto = $data['DSASSUNTO'];
            $assuntos = sizeof($data["ASSUNTO"]) > 0 ? $data["ASSUNTO"] : null;
            $dtfiltroitem = $data["DTFILTROITEM"] !== '' ? $data["DTFILTROITEM"] : null;
            if ($dtfiltroitem !== null) {
                $dtfiltroitem = \DateTime::createFromFormat('d/m/Y', $dtfiltroitem);
            }
            /** @var \Teknisa\Model\Teknisa\Entities\OmOqaItemAssunto[] $omOQAItemAssunto */
            $omOQAItemAssunto = InstanceProvider::getRepository()->getOmOqaItemAssunto()->findBy(array('itemId' => $id));
            if ($assuntos !== null) {
                foreach ($assuntos as $assunto) {
                    $omOQAAssunto = InstanceProvider::getRepository()->getOmOqaAssunto()->findOneBy(array('id' => $assunto));
                    $assuntos = $this->getPaiIdAssuntos($omOQAAssunto->getPaiId(), $assuntos);
                }
                $assuntos = array_unique($assuntos);
                foreach ($omOQAItemAssunto as $itemAssunto) {
                    if (!in_array($itemAssunto->getAssuntoId(), $assuntos)) {
                        $this->entityManager->remove($itemAssunto);
                    }
                }
                $idsAssuntoPai = Utilities::getArrayByArrayObjectWithGet($omOQAItemAssunto, ['getAssuntoId']);
                foreach ($assuntos as $assunto) {
                    if (!in_array($assunto, $idsAssuntoPai)) {
                        $omOQAItemAssunto = InstanceProvider::getFactory()->getCadItem()->createItemAssunto($id, $assunto);
                        $this->entityManager->persist($omOQAItemAssunto);
                    }
                }
            } else {
                if ($omOQAItemAssunto) {
                    foreach ($omOQAItemAssunto as $idItemAssunto) {
                        $this->entityManager->remove($idItemAssunto);
                    }
                }
            }
            $omOQAItemRepository = InstanceProvider::getRepository()->getOmOqaItem()->findOneBy(array('id' => $id));
            if(empty($omOQAItemRepository)) {
                throw new \Exception('Id não encontrado no banco de dados para edição');
            }

            $pathFile =  $omOQAItemRepository->getFilename();
            $typeFile =  $omOQAItemRepository->getFiletype();
            $sizeFile = $omOQAItemRepository->getFilesize();
            $labelFile = $omOQAItemRepository->getFilelabel();

            if (!empty($data['DOCUMENT'])) {
                if($omOQAItemRepository->getFilename() !== null){
                    InstanceProvider::getFileUploaderServe()->deleteFile($omOQAItemRepository->getFilename());
                }
                InstanceProvider::getFileUploaderServe()->uploadFilesInRow($data, 'DOCUMENT', 'OM/OqueHaDeNovo');
                $typeFile = $data['DOCUMENT'][0]['type'];
                $labelFile = $data['DOCUMENT'][0]['name'];
                $sizeFile = $data['DOCUMENT'][0]['size'];
                $pathFile = $data['DOCUMENT'][0]['path'];
            } else if ($omOQAItemRepository->getFilename() !== null && empty($data['PDF_PATH'])) {
                InstanceProvider::getFileUploaderServe()->deleteFile($omOQAItemRepository->getFilename());
                $typeFile = null;
                $labelFile = null;
                $sizeFile = null;
                $pathFile = null;
            }

            $omOQAItem = InstanceProvider::getFactory()->getCadItem()->updateItem(
                $omOQAItemRepository, $id, $dsassunto, $dtfiltroitem, $omOQAItemRepository->getProdutoId(), $omOQAItemRepository->getPaiId(),
                $pathFile, $typeFile, $sizeFile, $labelFile);
            $this->entityManager->persist($omOQAItem);

            if ($assuntos !== null) {
                $this->atualizaAssuntosPai($omOQAItemRepository->getPaiId(), $assuntos);
            }
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function removeItem($id)
    {   
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try{
            $idsAssunto = InstanceProvider::getRepository()->getOmOqaItemAssunto()->findBy(array('itemId' => $id));
            if (!empty($idsAssunto)) {
                foreach ($idsAssunto as $omOQAItemAssuntoRepository) {
                    $this->entityManager->remove($omOQAItemAssuntoRepository);
                    $this->entityManager->flush();
                }
            }
            $this->removeFilhoItemRecursivo($id);
            $omOQAItemRepository = InstanceProvider::getRepository()->getOmOqaItem()->findOneBy(array('id' => $id));
            if (!empty($omOQAItemRepository)) {
                $this->entityManager->remove($omOQAItemRepository);
                $this->entityManager->flush();
            }
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function removeFilhoItemRecursivo($id) {
        $omOQAItemRepository = InstanceProvider::getRepository()->getOmOqaItem()->findBy(array('paiId' => $id));
        foreach ($omOQAItemRepository as $filho) {
            $idsAssunto = InstanceProvider::getRepository()->getOmOqaItemAssunto()->findBy(array('itemId' => $filho->getId()));
            if (!empty($idsAssunto)) {
                foreach ($idsAssunto as $omOQAItemAssuntoRepository) {
                    $this->entityManager->remove($omOQAItemAssuntoRepository);
                    $this->entityManager->flush();
                }
            }
            $this->removeFilhoItemRecursivo($filho->getId());
            $this->entityManager->remove($filho);
        }
    }

    public function downloadFile($params) {
        $filePath = $params['path'];
        $fileName = $params['name'];
        $fileType = $params['type'];
        $fileLink = $this->fileServerFilesUrl . $params['path'];
        $this->setParamsForDownload($fileType, null, $fileName, $fileLink, false);
        InstanceProvider::getFileUploaderServe()->deleteFile($filePath);
        exit;
    }

    public function setParamsForDownload($type, $size, $name, $fileName,$isAttachment = true)
    {
        header('Accept-Ranges: none');
        header("Content-Type: " . $type);
        if($size){
            header("Content-Length: " . $size);
        }

        if($isAttachment){
            header('Content-Disposition: attachment; filename="' . $name.'"');
        }else{
            header('Content-Disposition: inline;filename="' . $name.'"');
        }
        readfile($fileName);
    }
}
