<?php

namespace OrganizationsManagement\Util;

use Doctrine\DBAL\DriverManager;
use Doctrine\ORM\EntityManager as EntityManager;
use Zeedhi\Framework\ORM\ConnectionFactory as ORMConnectionFactory;

class DBConnection {
    private $config;
    private $eventManager;
    private $connectionParams;
    private $auxConnectionParams;
    private $auxNotSaasConnectionParams;

    public function __construct(array $connectionParams, array $auxConnectionParams, array $auxNotSaasConnectionParams,
                                $config = null, $eventManager = null) {
        $this->config = $config;
        $this->eventManager = $eventManager;
        $this->connectionParams = $connectionParams;
        $this->auxConnectionParams = $auxConnectionParams;
        $this->auxNotSaasConnectionParams = $auxNotSaasConnectionParams;
    }

    public function getOrgEntityManager($nrOrg, $isAuxConnection = false, $isNotSaasOrg = false) {
        if($isAuxConnection) {
            if($isNotSaasOrg) {
                $this->connectionParams = $this->auxNotSaasConnectionParams;
            } else {
                $this->connectionParams = $this->auxConnectionParams;
            }
        } else {
            if (isset($this->connectionParams['sessionMode'])) {
                $this->connectionParams['dbname'] = 'USR_ORG_' . $nrOrg;
            } else {
                $this->connectionParams['user'] = 'USR_ORG_' . $nrOrg;
                if ((isset($this->connectionParams['useVpd']) && $this->connectionParams['useVpd'] == 'true') &&
                    (!isset($this->connectionParams['vpdWithWallet']) || $this->connectionParams['vpdWithWallet'] != 'true')
                ) {
                    $this->connectionParams['password'] = $this->connectionParams['vpdPassword'];
                }
            }
        }
        
        if(empty($this->connectionParams["password"])) {
            $this->connectionParams["password"] = '';
        } else if($this->connectionParams["isEncrypted"] == 'true') {
            $this->connectionParams["password"] = ORMConnectionFactory::decrypt($this->connectionParams["password"]);
        }

        $connection = DriverManager::getConnection($this->connectionParams, $this->config, $this->eventManager);
        return EntityManager::create($connection, $this->config, $this->eventManager);
    }
} 