<?php
namespace OrganizationsManagement\Util;

use Zeedhi\Framework\DataSource\Configuration;
use Zeedhi\Framework\DataSource\Manager\Doctrine\NameProvider;

class NameProviderImpl implements NameProvider
{
    public function getClassName($tableName)
    {
        if($tableName == 'ZHU_ACCESS_CONTROL_RULE') {
            return 'ZeedhiUser\Model\AccessControl\Entities\ZhuAccessControlRule';
        } else if($tableName == 'ZH_PREFERENCE_LAYOUT') {
            return 'Zeedhi\Preferences\API\Model\Entities\ZhPreferenceLayout';
        } else if($tableName == 'ZH_PREFERENCE_FILTER') {
            return 'Zeedhi\Preferences\API\Model\Entities\ZhPreferenceFilter';
        } else if($tableName == 'ZH_PREFERENCE_VIEW') {
            return 'Zeedhi\Preferences\API\Model\Entities\ZhPreferenceView';
        } else if($tableName == 'ZH_USER_WIDGET') {
            return 'Zeedhi\Preferences\API\Model\Entities\ZhUserWidget';
        }
        return false;
    }

    public function getDataSourceByName($dataSourceName) {
        $dataSourcesSubModels = $this->getDataSources();
        $dataSourceDirectory = $dataSourcesSubModels[$dataSourceName];
        return Configuration::factoryFromFileLocation($dataSourceDirectory, $dataSourceName);
    }

    private function getDataSources() {
        $ds = DIRECTORY_SEPARATOR;
        $preferencesDataSources = $this->getPreferencesDataSources($ds);
        $moduleDataSources = $this->getModuleDataSources($ds);
        $omDataSources = $this->getDataSourceOm($ds);
        return array_merge($omDataSources, array_merge($preferencesDataSources, $moduleDataSources));
    }

    private function getDataSourceOm($ds) {
        $dataSources = file_get_contents(__DIR__.$ds.'..'.$ds.'..'.$ds.'..'.$ds.'omDataSourcesMapping.json');
        return json_decode($dataSources, true);
    }

    private function getModuleDataSources($ds) {
        $file = file_get_contents(__DIR__.$ds.'..'.$ds.'..'.$ds.'..'.$ds.'modulesDataSources.json');
        return json_decode($file, true);
    }

    private function getPreferencesDataSources() {
        $preferencesDataSourceDir = '../vendor/zeedhi/preferences/gen/datasources';
        return array(
            'filter'      => $preferencesDataSourceDir,
            'layout'      => $preferencesDataSourceDir,
            'view'        => $preferencesDataSourceDir,
            'user_widget' => $preferencesDataSourceDir
        );
    }
}
