import { IDictionary, Metadata } from '@zeedhi/core';
import { OMInstance } from '@/services/OMInstance';
import { Button, LoadingService, Table } from '@zeedhi/common';
import { Highlight } from '@zeedhi/zd-highlight-common';
import { OmDialogService } from '@/services/OmDialogService';
import Prism from 'prismjs';
import 'prismjs/components/prism-markup-templating.min';
import 'prismjs/components/prism-php.min';
import moment from 'moment';
import { TekRestDatasource } from '@zeedhi/teknisa-components-common';

export class AnaliseColetaController {
    public filterDate: any;

    public isVisibleGrid : boolean = true;

    public isVisibleHighlights = false;

    public languageVar = 'php';

    get frameAnaliseColetaIsVisible() {
        return !this.isVisibleGrid;
    }

    public openFilter({ component }: any) {
        const filterButton = Metadata.getInstance<Button>(`${component.name}_filterButton`);
        filterButton.click({} as Event, {} as HTMLElement);
    }

    public get addDatasourceScript() {
        const queries = Metadata.getInstances('SCRIPT');
        return queries.find((query: any) => query.parent.name === 'fifithRowHighlightScript');
    }

    public get organizationHeader() {
        const queries = Metadata.getInstances('ORGANIZATION_HEADER');
        return queries.find((query: any) => query.parent.name === 'ZdOrganizationHeader');
    }

    public get addDatasourceScriptCount() {
        const queries = Metadata.getInstances('SCRIPT_COUNT');
        return queries.find((query: any) => query.parent.name === 'fifithRowScriptCount');
    }

    public get addDatasourcePrepStatus() {
        const queries = Metadata.getInstances('PREPSTATUS');
        return queries.find((query: any) => query.parent.name === 'firstRowStatus');
    }

    public get addDatasourceIdTipoOrg() {
        const queries = Metadata.getInstances('IDTIPOORG');
        return queries.find((query: any) => query.parent.name === 'firstRowIdTipoOrg');
    }

    public get addDatasourceIsPrimary() {
        const queries = Metadata.getInstances('IS_PRIMARY');
        return queries.find((query: any) => query.parent.name === 'firstRowIdPrimaria');
    }

    public get addDatasourceNmOrgPrincipal() {
        const queries = Metadata.getInstances('NMORGPRINCIPAL');
        return queries.find((query: any) => query.parent.name === 'firstRowOrganizationPrimary');
    }

    public get addDatasourceOrganization() {
        const queries = Metadata.getInstances('ORGANIZATION');
        return queries.find((query: any) => query.parent.name === 'firstRowOrganization');
    }

    public get addDatasourceProduct() {
        const queries = Metadata.getInstances('PRODUCT');
        return queries.find((query: any) => query.parent.name === 'firstRowProduct');
    }

    public get addDatasourceVariableMeasure() {
        const queries = Metadata.getInstances('VARIABLE_MEASURE');
        return queries.find((query: any) => query.parent.name === 'FirstRowVariableMeasure');
    }

    public get addDatasourceService() {
        const queries = Metadata.getInstances('SERVICE');
        return queries.find((query: any) => query.parent.name === 'SecondRowService');
    }

    public get addDatasourceContractCode() {
        const queries = Metadata.getInstances('CONTRACT_CODE');
        return queries.find((query: any) => query.parent.name === 'SecondRowContractCode');
    }

    public get addDatasourceData() {
        const queries = Metadata.getInstances('DATA');
        return queries.find((query: any) => query.parent.name === 'SecondRowData');
    }

    public get addDatasourceLastSuccessfulCollect() {
        const queries = Metadata.getInstances('LAST_SUC_COLLECT');
        return queries.find((query: any) => query.parent.name === 'SecondRowDataLastSucCollect');
    }

    public get addDatasourceStringCon() {
        const queries = Metadata.getInstances('DSSTRINGCONEXAO');
        return queries.find((query: any) => query.parent.name === 'ThirdRowStringCon');
    }

    public get addDatasourceWallet() {
        const queries = Metadata.getInstances('IDUTWALLET');
        return queries.find((query: any) => query.parent.name === 'ThirdRowWallet');
    }

    public get addDatasourceCollector() {
        const queries = Metadata.getInstances('IDCOLETOR');
        return queries.find((query: any) => query.parent.name === 'ThirdRowCollector');
    }

    public get addDatasourceTpColeta() {
        const queries = Metadata.getInstances('IDTPCOLETA');
        return queries.find((query: any) => query.parent.name === 'ThirdRowTpColeta');
    }

    public get addDatasourceDtIniCompetencia() {
        const queries = Metadata.getInstances('DTINICOMPET');
        return queries.find((query: any) => query.parent.name === 'ThirdTemp1RowDtIniCompet');
    }

    public get addDatasourceDtFinCompetencia() {
        const queries = Metadata.getInstances('DTFINCOMPET');
        return queries.find((query: any) => query.parent.name === 'ThirdTemp1RowDtFinCompet');
    }

    public get addDatasourceQtContratual() {
        const queries = Metadata.getInstances('QTCONTRATUAL');
        return queries.find((query: any) => query.parent.name === 'ThirdTemp1RowQtContratual');
    }

    public get addDatasourceQtMedida() {
        const queries = Metadata.getInstances('QTMEDIDA');
        return queries.find((query: any) => query.parent.name === 'ThirdTemp1RowQtMedida');
    }

    public get addDatasourceQtFaturada() {
        const queries = Metadata.getInstances('QTFATURADA');
        return queries.find((query: any) => query.parent.name === 'ThirdTemp1RowQtFaturada');
    }

    public get addDatasourceDsObsColeta() {
        const queries = Metadata.getInstances('DSOBSCOLETA');
        return queries.find((query: any) => query.parent.name === 'fourthRowDsObsColeta');
    }

    public get addDatasourceIdStatus() {
        const queries = Metadata.getInstances('IDSTATUS');
        return queries.find((query: any) => query.parent.name === 'fourthRowStatus');
    }

    public get addDatasourceCollapseCardScriptCountPHP() {
        const queries = Metadata.getInstances('SCRIPT_COUNT');
        return queries.find((query: any) => query.parent.name === 'collapseCardScriptCountPHP');
    }

    public get addDatasourceCollapseCardScriptPHP() {
        const queries = Metadata.getInstances('SCRIPT');
        return queries.find((query: any) => query.parent.name === 'collapseCardHighlightScriptPHP');
    }

    public get addDatasourceCollapseCardScriptCountSQL() {
        const queries = Metadata.getInstances('SCRIPT_COUNT');
        return queries.find((query: any) => query.parent.name === 'collapseCardScriptCountSQL');
    }

    public get addDatasourceCollapseCardScriptSQL() {
        const queries = Metadata.getInstances('SCRIPT');
        return queries.find((query: any) => query.parent.name === 'collapseCardHighlightScriptSQL');
    }

    public getPrimaryValue(value: string) {
        return value === 'P' ? 'Primária' : 'Secundária';
    }

    public getIdTipoOrgValue(value: string) {
        if (value === 'S') return 'SaaS';
        if (value === 'H') return 'Hosting';
        return 'Local';
    }

    public getStatusValue(value: string) {
        const allStatus = {
            P: 'Base Preparada',
            D: 'Base Bloqueada',
            Z: 'Base Preparada (Zmart Incluso)',
            C: 'Gerar Estrutura',
            Q: 'Gerar Dados Iniciais',
            S: 'Geração de Estrutura Solicitada',
            I: 'Organização Inativada',
            R: 'Organização Inativada - Base Removida',
            N: 'Preparação Pendente (Local/Hosting)',
            L: 'Organização Local/Hosting Preparada',
            Y: 'Organização Local/Hosting Preparada (Zmart Incluso)',
            J: 'Organização Local/Hosting Inativada',
            O: 'Preparação Pendente (Secundária)',
            M: 'Organização Secundária Preparada',
            X: 'Organização Secundária Preparada (Zmart Incluso)',
            K: 'Organização Secundária Inativada',
        };
        return value ? allStatus[value] : '';
    }

    public getIdstatus(value: string) {
        const allStatus = {
            E: 'Erro (Coleta Automática)',
            W: 'Alerta (Parametrização)',
            A: 'Alerta (Coleta Automática)',
            O: 'Coleta OK',
            X: 'Coleta OK (Erros em coletas Posteriores)',
        };
        return value ? allStatus[value] : '';
    }

    public async organizationGridRowClick(params) {
        this.isVisibleGrid = false;
        const fifthTable = Metadata.getInstance<Table>('fifthRow');
        const thirdTable = Metadata.getInstance<Table>('thirdRow');
        const lastSucCollect = Metadata.getInstance<Table>('SecondRowDataLastSucCollect');
        const SQLCollapseCard = Metadata.getInstance<Table>('SQL_Sub_Scripts');
        const PHPCollapseCard = Metadata.getInstance<Table>('PHP_Sub_Scripts');
        try {
            console.warn(params.row);
            LoadingService.show();
            this.addDatasourcePrepStatus.text = this.getStatusValue(params.row.PREPSTATUS);
            this.addDatasourceIdTipoOrg.text = this.getIdTipoOrgValue(params.row.IDTIPOORG);
            this.addDatasourceIsPrimary.text = this.getPrimaryValue(params.row.IS_PRIMARY);
            this.addDatasourceIdStatus.text = this.getIdstatus(params.row.IDSTATUS);
            this.addDatasourceNmOrgPrincipal.text = params.row.NMORGPRINCIPAL;
            this.addDatasourceOrganization.text = params.row.ORGANIZACAO;
            this.organizationHeader.text = params.row.NMPARCEIRO;
            this.addDatasourceVariableMeasure.text = params.row.DSTPMEDIDASERV;
            this.addDatasourceProduct.text = params.row.NMELEMENTO;
            this.addDatasourceService.text = params.row.NMSERVORCA;
            this.addDatasourceContractCode.text = params.row.CONTRATO;
            this.addDatasourceData.text = params.row.DTINCLUSAO;
            this.addDatasourceLastSuccessfulCollect.text = params.row.DTULTCOLETASUC;
            this.addDatasourceDsObsColeta.text = params.row.DSOBSCOLETA;
            this.addDatasourceDtIniCompetencia.text = params.row.DTINICOMPET;
            this.addDatasourceDtFinCompetencia.text = params.row.DTFINCOMPET;
            this.addDatasourceQtContratual.text = params.row.QTCONTRATUAL;
            this.addDatasourceQtMedida.text = params.row.QTMEDIDA;
            this.addDatasourceQtFaturada.text = params.row.QTFATURADA;
            this.addDatasourceStringCon.text = params.row.DSSTRINGCONEXAO;
            this.addDatasourceCollector.text = params.row.IDCOLETOR;
            this.addDatasourceTpColeta.text = params.row.IDTPCOLETA;
            this.addDatasourceScriptCount.code = params.row.SCRIPT_COUNT;
            this.addDatasourceScript.code = params.row.SCRIPT_LOG;
            this.addDatasourceWallet.text = params.row.IDUTWALLET;
            thirdTable.isVisible = true;
            if (params.row.IDUTWALLET === 'S') {
                this.addDatasourceWallet.text = 'Sim';
            } else {
                this.addDatasourceWallet.text = 'Não';
            }
            if (params.row.IDTPERRO === 'E' || params.row.IDTPERRO === 'A') {
                this.addDatasourceStringCon.text = params.row.DSSTRINGCONEXAO;
                this.addDatasourceCollector.text = params.row.IDCOLETOR;
                this.addDatasourceTpColeta.text = params.row.IDTPCOLETA;
                this.addDatasourceScriptCount.code = params.row.SCRIPT_COUNT;
                this.addDatasourceScript.code = params.row.SCRIPT_LOG;
                this.addDatasourceWallet.text = params.row.IDUTWALLET;
                if (params.row.IDUTWALLET === 'S') {
                    this.addDatasourceWallet.text = 'Sim';
                } else {
                    this.addDatasourceWallet.text = 'Não';
                }
                fifthTable.isVisible = true;
                thirdTable.isVisible = true;
                lastSucCollect.isVisible = true;
                PHPCollapseCard.isVisible = false;
                SQLCollapseCard.isVisible = false;
                this.isVisibleHighlights = false;
                if (params.row.IDTPSCRIPT === 'SQL') {
                    this.addDatasourceScript.language = 'sql';
                } else {
                    this.addDatasourceScript.language = 'php';
                }
            } else {
                this.addDatasourceCollapseCardScriptCountPHP.code = params.row.SCRIPT_COUNT_PHP;
                this.addDatasourceCollapseCardScriptPHP.code = params.row.SCRIPT_LOG_PHP;
                this.addDatasourceCollapseCardScriptCountSQL.code = params.row.SCRIPT_COUNT_SQL;
                this.addDatasourceCollapseCardScriptSQL.code = params.row.SCRIPT_LOG_SQL;
                fifthTable.isVisible = false;
                thirdTable.isVisible = true;
                lastSucCollect.isVisible = false;
                PHPCollapseCard.isVisible = true;
                SQLCollapseCard.isVisible = true;
                this.isVisibleHighlights = true;
            }
            LoadingService.hide();
        } catch (e) {
            LoadingService.hide();
            OmDialogService.showErrorBackend(e);
        }
    }

    get isVisibleHighLights(): boolean {
        return this.isVisibleHighlights;
    }

    get isVisibleDbVersionCard(): boolean {
        return !this.isVisibleGrid;
    }

    public closeButton() {
        this.isVisibleGrid = true;
    }

    public loadLanguage = this.loadLanguageAux.bind(this);

    public loadLanguageAux() {
        const query = this.addDatasourceScript;
        if (query) {
            const { code, language } = query;
            return Prism.highlight(code, Prism.languages[language], language);
        }
        return '';
    }

    public loadLanguageCollapseCardScriptPHP = this.loadLanguageAuxCollapseCardScriptPHP.bind(this);

    public loadLanguageAuxCollapseCardScriptPHP() {
        const query = this.addDatasourceCollapseCardScriptPHP;
        if (query) {
            const { code, language } = query;
            return Prism.highlight(code, Prism.languages[language], language);
        }
        return '';
    }

    public loadLanguageCollapseCardCountPHP = this.loadLanguageAuxCollapseCardCountPHP.bind(this);

    public loadLanguageAuxCollapseCardCountPHP() {
        const query = this.addDatasourceCollapseCardScriptCountPHP;
        if (query) {
            const { code, language } = query;
            return Prism.highlight(code, Prism.languages[language], language);
        }
        return '';
    }

    public loadLanguageCollapseCardScriptSQL = this.loadLanguageAuxCollapseCardScriptSQL.bind(this);

    public loadLanguageAuxCollapseCardScriptSQL() {
        const query = this.addDatasourceCollapseCardScriptSQL;
        if (query) {
            const { code, language } = query;
            return Prism.highlight(code, Prism.languages[language], language);
        }
        return '';
    }

    public loadLanguageCollapseCardCountSQL = this.loadLanguageAuxCollapseCardCountSQL.bind(this);

    public loadLanguageAuxCollapseCardCountSQL() {
        const query = this.addDatasourceCollapseCardScriptCountSQL;
        if (query) {
            const { code, language } = query;
            return Prism.highlight(code, Prism.languages[language], language);
        }
        return '';
    }

    public loadLanguageCount = this.loadLanguageAuxCount.bind(this);

    public loadLanguageAuxCount() {
        const query = this.addDatasourceScriptCount;
        if (query) {
            const { code, language } = query;
            return Prism.highlight(code, Prism.languages[language], language);
        }
        return '';
    }

    public loadLanguageExpression() {
        const { code, language } = Metadata.getInstance<Highlight>('expression');
        return Prism.highlight(code, Prism.languages[language], language);
    }

    public onCreatedCollectionErrorsGrid() {
        const date = new Date();
        const dtIniCompet = new Date(date.getFullYear(), date.getMonth(), 1);
        const today = moment().format('DD/MM/YYYY');
        const grid = OMInstance.getInstanceTekGrid('analise-coleta-grid');
        (grid.datasource as TekRestDatasource).dynamicFilter = {
            DTINICOMPET: [{
                relation: 'AND',
                operation: 'EQUALS',
                value: this.dataFormatada(dtIniCompet),
            }],
            DTFINCOMPET: [{
                relation: 'AND',
                operation: 'EQUALS',
                value: today,
            }],
        };
        grid.datasource.get();
    }

    public dataFormatada(data: any) {
        const dia = data.getDate().toString();
        const diaF = (dia.length === 1) ? `${'0'}${dia}` : dia;
        const mes = (data.getMonth() + 1).toString();
        const mesF = (mes.length === 1) ? `${'0'}${mes}` : mes;
        const anoF = data.getFullYear();
        return `${diaF}${'/'}${mesF}${'/'}${anoF}`;
    }

    public rowStyleConditions(row: IDictionary) {
        if (row.IDSTATUS === 'O') {
            return {
                cssClass: '',
            };
        }

        if ((row.IDSTATUS === 'W') || (row.IDSTATUS === 'A')) {
            return {
                cssClass: 'idstatusWarning',
            };
        }

        if (row.IDSTATUS === 'X') {
            return {
                cssClass: 'idstatusLaterErrors',
            };
        }

        if (row.IDSTATUS === 'E') {
            return {
                cssClass: 'idstatusErro',
            };
        }

        return { cssClass: '' };
    }

    public getConditionalCssClass({ row }: IDictionary<any>) {
        let color;

        if (row.IDSTATUS === 'E') {
            color = { color: '#FFFFFF' };
        } else if (row.IDSTATUS === 'A') {
            color = { color: '#008000' };
        } else if (row.IDSTATUS === 'O') {
            color = { color: '#FF0000' };
        }

        return color;
        /*
        switch (row.IDSTATUS) {
            case 'O':
                return 'idstatus';
            case 'A':
                return 'idstatusLaterErrors';
            default:
                return 'idstatusErro';
        }
        if (param.row.IDSTATUS === 'E') return 'idstatusErro';
        if (param.row.IDSTATUS === 'A') return 'idstatusWarning';
        if (param.row.IDSTATUS === 'X') return 'idstatus';
        if (param.row.IDSTATUS === 'O') return 'idstatusLaterErrors';
        return 'idstatusWarningCollect';
        */
    }
}
