import { DialogServiceCustom } from '@/services/DialogServiceCustom';
import { Utilities } from '@/util/Utilities';
import { DialogService, Form, LoadingService, Modal, ModalService, Select, SelectTree, SelectTreeMultiple, TextInput, Date } from '@zeedhi/common';
import { Http, Metadata, Config, Validation } from '@zeedhi/core';
import { ITekGridLayout, TekRestDatasource, TekTreeGrid } from '@zeedhi/teknisa-components-common';
import Messages from '@/services/Messages';

const invalidDate = () => {
    const message = 'É necessário que seja um intervalo entre duas datas';
    return (value: string[]) => {
        if (value.length === 1) {
            return message;
        }
        return true;
    };
};

const intervalDate = () => {
    const message = 'O intervalo deve ser de no máximo 1 ano';
    return (value: string[]) => {
        if (value.length === 2) {
            const [day1, month1, year1] = value[0].split('/').map(Number);
            const [day2, month2, year2] = value[1].split('/').map(Number);
            const days1 = year1 * 365 + month1 * 30 + day1;
            const days2 = year2 * 365 + month2 * 30 + day2;
            const intervalInDays = Math.abs(days2 - days1);
            if (intervalInDays > 365) {
                return message;
            }
        }
        return true;
    };
};

Validation.register('invalidDate', invalidDate);
Validation.register('intervalDate', intervalDate);

export class OqueHaDeNovoController {
    public visibleGrid: boolean = true;

    public visibleFrame: boolean = false;

    public denieDeleteButton: boolean = true;

    public denieDeleteButtonEdit: boolean = true;

    public currentRow: any = {};

    public currentFile: any = {};

    public currentFileEdit: any = {};

    private docSrc = '';

    private docSrcEdit = '';

    private doomIframe: any;

    public ModalOqueHaDeNovoAdd!: Modal;

    public ModalOqueHaDeNovoEdit!: Modal;

    public ModalUploadDocOQA!: Modal;

    public ModalUploadDocOQAEdit!: Modal;

    public baseUrl = '';

    public async openModalUploadDocOQA() {
        this.ModalUploadDocOQA = await ModalService.createFromJson('/oqueHaDeNovo/modal/modalUploadDocOQA', true);
        this.ModalUploadDocOQA.show();
    }

    public closeModalUploadDocOQA() {
        this.ModalUploadDocOQA.destroy();
    }

    public async openModalUploadDocOQAEdit() {
        this.ModalUploadDocOQAEdit = await ModalService.createFromJson('/oqueHaDeNovo/modal/modalUploadDocOQAEdit', true);
        this.ModalUploadDocOQAEdit.show();
    }

    public closeModalUploadDocOQAEdit() {
        this.ModalUploadDocOQAEdit.destroy();
    }

    public setDocSrc() {
        const doomIframe = document.getElementById('iframeDocIdAdd') as HTMLIFrameElement;
        doomIframe.src = this.docSrc;
    }

    public setDocSrcEdit() {
        const doomIframe = document.getElementById('iframeDocId') as HTMLIFrameElement;
        doomIframe.src = this.docSrcEdit;
    }

    public async deleteFileOQA() {
        LoadingService.show();
        this.denieDeleteButton = true;
        this.docSrc = '';
        this.currentFile = {};
        Metadata.getInstance<Date>('modalOqueHaDeNovoAddForm-DTFILTROITEM').removeValidation('required');
        await this.setDocSrc();
        LoadingService.hide();
    }

    public async deleteFileOQAEdit() {
        LoadingService.show();
        this.denieDeleteButtonEdit = true;
        this.docSrcEdit = '';
		this.currentRow.pdfPath = '';
        this.currentFileEdit = {};
        Metadata.getInstance<Date>('modalOqueHaDeNovoEdit-DTFILTROITEM').removeValidation('required');
        await this.setDocSrcEdit();
        LoadingService.hide();
    }

    public async addNewRow() {
        this.ModalOqueHaDeNovoAdd = await ModalService.createFromJson('/oqueHaDeNovo/modal/modalOqueHaDeNovoAdd', true);
        this.ModalOqueHaDeNovoAdd.show();
    }

    public closeModalOqueHaDeNovoAdd() {
        this.ModalOqueHaDeNovoAdd.destroy();
    }

    public backToPageMain() {
        this.visibleGrid = true;
        this.visibleFrame = false;
    }

    get tituloDetail() {
        return this.currentRow.NAME;
    }

    public async rowClickFunction({ row }: any) {
        if (row.IS_PRODUCT === 'N') {
            this.currentRow = row;
            this.ModalOqueHaDeNovoEdit = await ModalService.createFromJson('/oqueHaDeNovo/modal/modalOqueHaDeNovoEdit', true);
            this.ModalOqueHaDeNovoEdit.show();
        }
    }

    public closeModalOqueHaDeNovoEdit() {
        this.ModalOqueHaDeNovoEdit.destroy();
    }

    public async editRow() {
        Metadata.getInstance<TextInput>('modalOqueHaDeNovoEdit-DSASSUNTO').value = this.currentRow.NAME;
        Metadata.getInstance<Select>('modalOqueHaDeNovoEdit-produto').value = this.currentRow.PRODUTO_ID;
        Metadata.getInstance<Date>('modalOqueHaDeNovoEdit-DTFILTROITEM').value = Utilities.formatDateHoursString(this.currentRow.DTFILTROITEM);
        const idsAssunto = await Http.post('/getSubjectsItem', {
            ID: this.currentRow.ID,
        });
        Metadata.getInstance<SelectTreeMultiple>('modalOqueHaDeNovoEdit-ASSUNTO').setValue(idsAssunto.data);
        LoadingService.show();
        const fileItem = await Http.post('/getPathFileItem', {
            ID: this.currentRow.ID,
        });
        if (fileItem.data.pdfPath) {
            this.docSrcEdit = fileItem.data.pdfPath;
			this.currentRow.pdfPath = this.docSrcEdit;
			this.denieDeleteButtonEdit = false;
            Metadata.getInstance<Date>('modalOqueHaDeNovoEdit-DTFILTROITEM').addValidation('required');
        } else {
            this.docSrcEdit = '';
			this.currentRow.pdfPath = '';
            this.denieDeleteButtonEdit = true;
        }
        await this.setDocSrcEdit();
        LoadingService.hide();
    }

    public changeItemProduto() {
        const item = Metadata.getInstance<SelectTree>('modalOqueHaDeNovoAddForm-item');
        const idProduto = Metadata.getInstance<Select>('modalOqueHaDeNovoAddForm-produto').value;
        if (item.datasource instanceof TekRestDatasource) {
            item.nodes = [];
            item.datasource.data = [];
            item.datasource.setFilter({
                PRODUTO_ID: idProduto,
            });
            item.setValue(null);
        }
    }

    public async saveModalAddOqueHaDeNovo() {
        const form = Metadata.getInstance<Form>('modalOqueHaDeNovoAddForm');
        const grid = Metadata.getInstance<TekTreeGrid>('oqueHaDeNovoTreeGrid');
        let paiId = null;
        if (Metadata.getInstance<SelectTree>('modalOqueHaDeNovoAddForm-item').value !== null) {
            paiId = Metadata.getInstance<SelectTree>('modalOqueHaDeNovoAddForm-item').value.ID;
        }
        const file = this.currentFile;
        if (Utilities.validateForm(form)) {
            LoadingService.show();
            Http.post('/saveModalAddOqueHaDeNovo', {
                DSASSUNTO: Metadata.getInstance<TextInput>('modalOqueHaDeNovoAddForm-DSASSUNTO').value,
                ASSUNTO: Metadata.getInstance<SelectTreeMultiple>('modalOqueHaDeNovoAddForm-ASSUNTO').value,
                PRODUTO_ID: Metadata.getInstance<Select>('modalOqueHaDeNovoAddForm-produto').value,
                ITEM_SUPERIOR: paiId,
                DTFILTROITEM: Metadata.getInstance<Date>('modalOqueHaDeNovoAddForm-DTFILTROITEM').value,
                DOCUMENT: file,
            }).then(() => {
                this.closeModalOqueHaDeNovoAdd();
                grid.datasource.setFilter({
                    PARENT: null,
                });
                grid.datasource.get();
                this.denieDeleteButton = true;
                this.currentFile = {};
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
	}

    public async uploadDocFrame() {
        LoadingService.show();
        const file = await Utilities.buildArrayToUpload('formModalUploadDocOQA-ARQUIVO');
        this.currentFile = file;
        try {
            const response = await Http.post('/uploadDocumentOQA', {
                DOCUMENT: file,
            });
            if (response.data[0]) {
                this.baseUrl = Config.endPoint;
                const downloadUri = `${this.baseUrl}/ViewPDF?path=${response.data[0].path}&type=${response.data[0].type}&name=${''}`;
                this.docSrc = downloadUri;
                this.denieDeleteButton = false;
                Metadata.getInstance<Date>('modalOqueHaDeNovoAddForm-DTFILTROITEM').addValidation('required');
            } else {
                this.docSrc = '';
                this.denieDeleteButton = true;
            }
            await this.setDocSrc();
            this.closeModalUploadDocOQA();
            LoadingService.hide();
        } catch (e) {
            Utilities.showErrorMessage(e);
            LoadingService.hide();
        }
        LoadingService.hide();
    }

    public async uploadDocFrameEdit() {
        LoadingService.show();
        const file = await Utilities.buildArrayToUpload('formModalUploadDocOQAEdit-ARQUIVO');
        this.currentFileEdit = file;
        try {
            const response = await Http.post('/uploadDocumentOQA', {
                DOCUMENT: file,
            });
            if (response.data[0]) {
                this.baseUrl = Config.endPoint;
                const downloadUri = `${this.baseUrl}/ViewPDF?path=${response.data[0].path}&type=${response.data[0].type}&name=${''}`;
                this.docSrcEdit = downloadUri;
                this.denieDeleteButtonEdit = false;
                Metadata.getInstance<Date>('modalOqueHaDeNovoEdit-DTFILTROITEM').addValidation('required');
            } else {
                this.docSrc = '';
                this.denieDeleteButtonEdit = true;
            }
            await this.setDocSrcEdit();
            this.closeModalUploadDocOQAEdit();
            LoadingService.hide();
        } catch (e) {
            Utilities.showErrorMessage(e);
            LoadingService.hide();
        }
        LoadingService.hide();
    }

    public async saveModalAddOqueHaDeNovoEdit() {
        const form = Metadata.getInstance<Form>('modalOqueHaDeNovoEdit-form');
        const grid = Metadata.getInstance<TekTreeGrid>('oqueHaDeNovoTreeGrid');
        const file = this.currentFileEdit;
        if (Utilities.validateForm(form)) {
            LoadingService.show();
            Http.post('/updateItemOqueHaDeNovo', {
                ID: this.currentRow.ID,
                DSASSUNTO: Metadata.getInstance<TextInput>('modalOqueHaDeNovoEdit-DSASSUNTO').value,
                ASSUNTO: Metadata.getInstance<SelectTreeMultiple>('modalOqueHaDeNovoEdit-ASSUNTO').value,
                DTFILTROITEM: Metadata.getInstance<Date>('modalOqueHaDeNovoEdit-DTFILTROITEM').value,
				PDF_PATH: this.currentRow.pdfPath,
                DOCUMENT: file,
            }).then(() => {
                grid.datasource.setFilter({
                    PARENT: null,
                });
                grid.datasource.get();
                this.closeModalOqueHaDeNovoEdit();
                this.currentFileEdit = {};
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
	}

    public async beforeDeleteItems(params: any) {
		params.event.preventDefault();
        const buttonYes = [
            {
                name: Utilities.getNameDynamic('yes-'),
                component: 'ZdButton',
                label: 'Sim',
                events: {
                    click: () => {
                        DialogService.hide();
                        this.doDeleteItems(params);
                    },
                },
            },
        ];
        DialogServiceCustom.confirmMessage(Messages.DELETE_DEFAULT_TREE, buttonYes);
	}

    public async doDeleteItems(params: any) {
		LoadingService.show();
		try {
			await Http.post('/removeItem', {
				data: params.component.selectedRows,
			});
            params.component.datasource.setFilter({
                PAI_ID: null,
            });
			params.component.selectedRows = [];
			params.component.datasource.get();
		} catch (e) {
			DialogServiceCustom.showErrorBackend(e);
		}
		LoadingService.hide();
	}

    public notProductSelected(component: any) {
        if (component.row.IS_PRODUCT === 'Y') {
            component.component.selectedRows = [];
        }
    }

    public async filterOnMounted() {
        const oqueHaDeNovoTreeGrid = Metadata.getInstance<TekTreeGrid>('oqueHaDeNovoTreeGrid');
		if (oqueHaDeNovoTreeGrid.datasource instanceof TekRestDatasource) {
			const dynamicFilter = {
				DTFILTROITEM: [
					{
						operation: 'BETWEEN',
						relation: 'AND',
						value: Utilities.getDates(),
					},
				],
			};
			const defaultLayout: ITekGridLayout = {
				name: 'Padão',
				dynamicFilter,
			};
			oqueHaDeNovoTreeGrid.layoutOptions?.updateDefaultLayout(defaultLayout);
		}
    }
}
