import { AlertService, LoadingService, TreeGridEditable } from '@zeedhi/common';
import { Http, Metadata } from '@zeedhi/core';

export class VersionRegistrationController {
    public reloadGrid({ component }:any) {
        const grid = component.parent;
        grid.reload();
    }

    public async cancelEditing({ event }:any) {
        event.preventDefault();
        const grid = Metadata.getInstance<TreeGridEditable>('versionregs-grid');
        await grid.cancelEditedRows();
    }

    public async saveEditing({ event }:any) {
        event.preventDefault();
        LoadingService.show('Atualizando...');
        const grid = Metadata.getInstance<TreeGridEditable>('versionregs-grid');
        const modifiedItems = grid.getEditedRows() as Object;
        const requestBody = [] as any;
        /* eslint-disable */
        for (const item in modifiedItems) {
            const element = modifiedItems[item];
            requestBody.push({
                nmrVerObjBd: element.NRVEROBJBD,
                originalData: {
                    CDARVPROD: element.CDARVPROD,
                    CDPATCHVERPR: element.CDPATCHVERPR,
                    CDPRODGRUP: element.CDPRODGRUP,
                    CDRELVERPROD: element.CDRELVERPROD,
                    CDVERPRODUTO: element.CDVERPRODUTO,
                    CDVERSAOPRODUTO: element.CDVERSAOPRODUTO,
                }
            })
        }
        /* eslint-enable */
        await Http.post('/versionRegistration/save', requestBody).then((value) => {
            AlertService.show({
                name: 'update-alert',
                text: value.data.success,
                color: 'success',
            });
            grid.reload();
            grid.cancelEditedRows();
        }, (reason) => {
            AlertService.show({
                name: 'update-alert',
                text: reason.message,
                color: 'error',
            });
        }).finally(() => {
            LoadingService.hide();
        });
    }
}
